`timescale 1ns / 1ps  // 22.9.2015
// with SRAM, byte access, flt.-pt., and gpio
// PS/2 mouse and network 7.1.2014 PDR
// Target device: Riskfive board
// FPGA:    ARTIX-7
// Modified by: Vedant Karia
/* Modifications:
 Removed the Asych SRAM signals and integrated the ZBT memory
 Removed switches from thw design 
 Removed LED bits from design
*/
module RISC5Top(
  input         CLK50M,
  input         btn,
  input         RxD,   // RS-232
  output        TxD,
  output        zbtrw,   
  output [3:0]  SRbe,      //byte enables for ZBT mem
  output [19:0] zbtadr,    // Address bits for ZBT
  output        zbtadv,
  output        zbtclk,    //ZBT memory clk
  inout  [31:0] zbtdata,   // ZBT data pins
  input  [1:0]  MISO,      // SPI - SD card & network
  output [1:0]  SCLK, MOSI, 
  output [1:0]  SS,
  output        hsync, vsync, // video controller
  output        VGA_blank,  
  output        VGA_sync, 
  output        VGA_pix_clk,  // 75Mhz clk from Video core pclk
  output [23:0] RGB,
  input         PS2C, PS2D,    // keyboard
  inout         msclk, msdat);

// IO addresses for input / output
// 0  milliseconds / --
// 2  RS-232 data / RS-232 data (start)
// 3  RS-232 status / RS-232 control
// 4  SPI data / SPI data (start)
// 5  SPI status / SPI control
// 6  PS2 keyboard / --
// 7  mouse / --
// 8  general-purpose I/O data
// 9  general-purpose I/O tri-state control

reg rst;
wire[23:0] adr;
wire [3:0] iowadr; // word address
wire [31:0] inbus, inbus0;  // data to RISC core
wire [31:0] outbus;  // data from RISC core
wire rd, wr, ben, ioenb, dspreq;

wire [7:0] dataTx, dataRx, dataKbd;
wire rdyRx, doneRx, startTx, rdyTx, rdyKbd, doneKbd;
wire [27:0] dataMs;
reg bitrate;  // for RS232
wire limit;  // of cnt0

reg [7:0] Lreg;
reg [15:0] cnt0;
reg [31:0] cnt1; // milliseconds
wire [7:0]leds;
wire [31:0] spiRx;
wire spiStart, spiRdy;
reg [3:0] spiCtrl;
wire [17:0] vidadr;
reg [7:0] gpout, gpoc;
wire [7:0] gpin;

wire wr_enable;
wire SRce0, SRce1, SRbe0, SRbe1;
wire [19:0]SRadr;
wire  SRwe; 


//////////////////////////////////////////////////////////////////////    
      wire clkfbout, pllclk0, pllclk1, pllclk2, pllclk3;
      wire locked;
      wire pll_locked;
      wire clkpll, clkpll1 ; 
      wire pclk, clk_200_90, clk_200, clk;
      wire   clkin;   
     IBUFG  u1   (.I(CLK50M), .O(clkin));
      PLL_BASE # (
        .CLKIN_PERIOD(40),
        .CLKFBOUT_MULT(24),
        .CLKOUT0_DIVIDE(24),    // 25 Mhz
        .CLKOUT1_DIVIDE(8),     // 75 Mhz  
        .CLKOUT2_DIVIDE(3),     // 200 Mhz  
        .CLKOUT3_DIVIDE(3),     // 200 Mhz for the ZBT memory  
        .CLKOUT0_PHASE(0.0),
        .CLKOUT1_PHASE(0.0),
        .CLKOUT2_PHASE(90.0),
        .CLKOUT3_PHASE(30.0),
        .COMPENSATION("INTERNAL")
      ) pll_blk (
        .CLKFBOUT(clkfbout),
        .CLKOUT0(pllclk0),
        .CLKOUT1(pllclk1),
        .CLKOUT2(pllclk2),
        .CLKOUT3(pllclk3),
        .CLKOUT4(),
        .CLKOUT5(),
        .LOCKED(pll_locked),
        .CLKFBIN(clkfbout),
        .CLKIN(clkin),
        .RST(1'b0)
          );      
          BUFG clk25bufg(.I(pllclk0), .O(clk));  // 
          BUFG clk75bufg(.I(pllclk1), .O(pclk));  // 
          BUFG clk200bufg(.I(pllclk3), .O(clk_200));  // 
          BUFG clk200_90bufg(.I(pllclk2), .O(clk_200_90));  // 
///////////////////////////////////////////////////////////////////////

RISC5 riscx(.clk(clk), .rst(rst), .rd(rd), .wr(wr), .ben(ben), .stallX(dspreq),
   .adr(adr), .codebus(inbus0), .inbus(inbus), .outbus(outbus));
   
RS232R receiver(.clk(clk), .rst(rst), .RxD(RxD), .fsel(bitrate), .done(doneRx),
   .data(dataRx), .rdy(rdyRx));
RS232T transmitter(.clk(clk), .rst(rst), .start(startTx), .fsel(bitrate),
   .data(dataTx), .TxD(TxD), .rdy(rdyTx));
SPI spi(.clk(clk), .rst(rst), .start(spiStart), .dataTx(outbus),
   .fast(spiCtrl[2]), .dataRx(spiRx), .rdy(spiRdy),
 	.SCLK(SCLK[0]), .MOSI(MOSI[0]), .MISO(MISO[0] & MISO[1]));
VID vid(.clk(clk), .pclk(pclk), .req(dspreq), .inv(0),//swi[7]),
   .vidadr(vidadr), .viddata(inbus0), .RGB(RGB), .hsync(hsync), .vsync(vsync));
PS2 kbd(.clk(clk), .rst(rst), .done(doneKbd), .rdy(rdyKbd), .shift(),
   .data(dataKbd), .PS2C(PS2C), .PS2D(PS2D));
MouseP Ms(.clk(clk), .rst(rst), .msclk(msclk), .msdat(msdat), .out(dataMs));

// assigning the 200 Mhz clock to latch the data and control bits. mem_clk is connected to the ZBT clk. Phase difference given between the clk and mem_clk. 
ZBT_controller_P memory_ctrl (.clk(clk_200), .mem_clk(clk_200_90),.address(SRadr), .data_in(outbus),.be_n_0(byte_en[0]),.be_n_1(byte_en[1]), .be_n_2(byte_en[2]),
   .be_n_3(byte_en[3]),.wr_n(SRwe),.ZBT_ADV(zbtadv),.ZBT_wr(zbtrw),.ZBT_clk(zbtclk),.ZBT_data(zbtdata),.data_out(inbus0), .ZBT_address(zbtadr),.ZBT_byte_en(SRbe));

wire md, mc;
assign md = msdat;
assign mc = msclk;

//ila_0 debug (.clk(clk), .probe0(rst), .probe1(mc), .probe2(md), .probe3(PS2D));

assign iowadr = adr[5:2];
assign ioenb = (adr[23:6] == 18'h3FFFF);
assign inbus = ~ioenb ?  inbus0 :
   ((iowadr == 0) ? cnt1 :
    (iowadr == 1) ? {20'b0, 4'b0, 8'b0} :
    (iowadr == 2) ? {24'b0, dataRx} :
    (iowadr == 3) ? {30'b0, rdyTx, rdyRx} :
    (iowadr == 4) ? spiRx :
    (iowadr == 5) ? {31'b0, spiRdy} :
    (iowadr == 6) ? {3'b0, rdyKbd, dataMs} :
    (iowadr == 7) ? {24'b0, dataKbd} :
    (iowadr == 8) ? {24'b0, gpin} :
    (iowadr == 9) ? {24'b0, gpoc} : 0);
	 


assign VGA_pix_clk = pclk;
assign VGA_blank = 1;
assign VGA_sync = 1;

/////////// Byte enable signals used for Async SRAM/////////////////
//assign SRce0 = ben & adr[1];
//assign SRce1 = ben & ~adr[1];
//assign SRbe0 = ben & adr[0];
//assign SRbe1 = ben & ~adr[0];
////////////////////////////////////////////////////////////////////


wire [3:0]byte_en; // byte enable signals for ZBT memory 

assign byte_en = {(ben & (adr[1:0] != 2'b11)),
                  (ben & (adr[1:0] != 2'b10)),
                  (ben & (adr[1:0] != 2'b01)),
                  (ben & (adr[1:0] != 2'b00))}; // Byte enable only if ben is set by processor.
                  
                  
assign SRwe = dspreq ? 1'b1: ~(wr & clk);      // Setting SRwe bit only during the positive edge of the clk(25MHz clock).

//assign SRoe = wr;     // ZBT Output enable is tied low on the board which automatically 
//assign SRbe = {SRbe1, SRbe0, SRbe1, SRbe0};
assign SRadr = dspreq ? {2'b00, vidadr} :{ 2'b00, adr[19:2]};       // SRadr for assigning the address from Video core or the main processor

assign dataTx = outbus[7:0];    // Serial data transmission assignment 
assign startTx = wr & ioenb & (iowadr == 2);
assign doneRx = rd & ioenb & (iowadr == 2);
assign limit = (cnt0 == 24999);
assign leds = Lreg;
assign spiStart = wr & ioenb & (iowadr == 4);
assign SS = ~spiCtrl[1:0];  //active low slave select
assign MOSI[1] = MOSI[0], SCLK[1] = SCLK[0];// NEN = spiCtrl[3];
assign doneKbd = rd & ioenb & (iowadr == 7);
//assign SDled = spiCtrl[0];

always @(posedge clk)
begin
  rst <= ((cnt1[4:0] == 0) & limit) ? btn : rst;
  Lreg <= ~rst ? 0 : (wr & ioenb & (iowadr == 1)) ? outbus[7:0] : Lreg;
  cnt0 <= limit ? 0 : cnt0 + 1;
  cnt1 <= cnt1 + limit;
  spiCtrl <= ~rst ? 0 : (wr & ioenb & (iowadr == 5)) ? outbus[3:0] : spiCtrl;
  bitrate <= ~rst ? 0 : (wr & ioenb & (iowadr == 3)) ? outbus[0] : bitrate;
  gpout <= (wr & ioenb & (iowadr == 8)) ? outbus[7:0] : gpout;
  gpoc <= ~rst ? 0 : (wr & ioenb & (iowadr == 9)) ? outbus[7:0] : gpoc;
end

endmodule