`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Skutek Instrumentation
// Engineer: Vedant Karia.
// 
// Create Date: 12/04/2018 02:16:11 PM
// Design Name: ZBT memory chip controller
// Module Name: ZBT_controller_P
// Project Name: Oberon system on Riskfive
// Target Devices: Artix-7 FPGA
// Tool Versions: 2018.3
// Description: This reads the address put on the address pin and assigns to ZBT address pins. After two clock cycles of "clk" the data is written on the data bus. 
// address and the read/write bit are directly assigned to the pins but data is read or written after two clock cycle. 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module ZBT_controller_P(
    // fabric processor side
    // There are two clock going because "clk" is used to latching the data from processor to the ZBT chip data bus
    //"mem_clk" is the clock from top file which is assigned to the zbt chip clock
    input wire clk, // 200 MHz cklock controlling the block
    input wire mem_clk, // assigned to memory chip clock 200 MHz
    input wire [19:0]address,
    input wire [31:0]data_in,
    input wire be_n_0,
    input wire be_n_1,
    input wire be_n_2,
    input wire be_n_3,
    input wire wr_n,
    output wire [31:0]data_out,    
    // ZBT chip side
    output wire         ZBT_ADV,
    output reg       ZBT_wr,
    output wire         ZBT_clk,
    inout wire [31:0]   ZBT_data,
    output reg [19:0]   ZBT_address,
    output wire [3:0]   ZBT_byte_en
    );
   
reg [31:0]write_data, write_data1, write_data2;
reg  control1, control2, control;
wire [31:0] data_read;
assign ZBT_clk = mem_clk;                                   // assigning the 90degree phase shifted clock to the ZBT clk pin. mem_clock should be 90 shifted to the "clk". Done in the top file
assign ZBT_ADV = 0;                     
assign ZBT_byte_en = {be_n_3, be_n_2 , be_n_1 , be_n_0};    //Byte enable bits comming from processor. 
//assign ZBT_address = address; // assigning the address to the address bus at clock1.
//assign ZBT_wr      = wr_n;    // assigning the wr to the wr pin at clock1.


assign ZBT_data = ~control ? write_data : 32'hzzzzzzzz;     //Bidirectional bus if read the control is 1 and if write control is 0


assign data_read = ZBT_data;
assign data_out = control? data_read: data_out; 
      
always@(posedge clk)
begin
    ZBT_address <= address; // assigning the address to the address bus at clock1.
    ZBT_wr      <= wr_n;    // assigning the wr to the wr pin at clock1.

   write_data1 <= data_in;
   write_data2 <= write_data1;
   write_data  <= write_data1;  // assigning the wr to the wr pin at clock3.
   
   control1 <= wr_n;
   control2 <= control1;
   control <= control1;        // Changing the bus direction after two clock cycles.
end 
    
endmodule
